puts "Running data capture script..."

puts "Connecting to the processor..."
    connect mb mdm

puts "Downloading the Microblaze program..."
    rst
    stop
    rst
    dow ADIEvalBoard.elf
    run
    stop

puts "Configuring the ADC"
# Program the HDL driver
# Control Register
# Write
 set control [ expr  1 << 15]
# Sequence (the SEQ bit)
 set sequence 0
 set control [ expr $control + [ expr $sequence << 14 ] ]
# Channel ( 0 to 15 )
 set channel 0
 set control [ expr $control + [ expr $channel << 10 ] ]
# Power ( Normal = 3, Full shutdown = 2, Auto shutdown = 1, Auto standby = 0)
 set power 3
 set control [ expr $control + [ expr $power << 8 ] ]
# Shadow
 set shadow 0
 set control [ expr $control + [ expr $shadow << 7 ] ]
# WEAK / TRISTATE ( Weak = 1 , Tristate = 0)
 set weaktri 0
 set control [ expr $control + [ expr $weaktri << 6 ] ]
# Range ( 2xVRef = 0 , VRef = 1)
 set range 0
 set control [ expr $control + [ expr $range << 5 ] ]
# Coding ( 2's complement = 0, Straight binary = 1 )
 set coding 0
 set control [ expr $control + [ expr $coding << 4 ] ]
#clean unused variables
unset sequence
unset channel
unset power
unset shadow
unset weaktri

# Shadow Register ( uncomment the set command under the channel you want to be added )
# Channel 0
set control [ expr $control + [ expr 1 << 31 ] ]
# Channel 1
#set control [ expr $control + [ expr 1 << 30 ] ]
# Channel 2
#set control [ expr $control + [ expr 1 << 29 ] ]
# Channel 3
#set control [ expr $control + [ expr 1 << 28 ] ]
# Channel 4
#set control [ expr $control + [ expr 1 << 27 ] ]
# Channel 5
# set control [ expr $control + [ expr 1 << 26 ] ]
# Channel 6
# set control [ expr $control + [ expr 1 << 25 ] ]
# Channel 7
# set control [ expr $control + [ expr 1 << 24 ] ]
# Channel 8
# set control [ expr $control + [ expr 1 << 23 ] ]
# Channel 9
# set control [ expr $control + [ expr 1 << 22 ] ]
# Channel 10
# set control [ expr $control + [ expr 1 << 21 ] ]
# Channel 11
# set control [ expr $control + [ expr 1 << 20 ] ]
# Channel 12
# set control [ expr $control + [ expr 1 << 19 ] ]
# Channel 13
# set control [ expr $control + [ expr 1 << 18 ] ]
# Channel 14
# set control [ expr $control + [ expr 1 << 17 ] ]
# Channel 15
#set control [ expr $control + [ expr 1 << 16 ] ]
set configureAddr [mrd_var pCommand]
set configureAddr [format "0x%08x" $configureAddr]
mwr $configureAddr  [format "0x%08x" $control]
con
after 100
stop

puts "Starting the data acquisition..."
    set startAcqAddr [mrd_var pStartAcquisition]
    set startAcqAddr [format "0x%08x" $startAcqAddr]
    mwr $startAcqAddr 1
    con
    after 1000
    stop

puts "Reading data from the system..."
    set rxDataAddr [mrd_var pRxData]
    set rxDataAddr [format "0x%08x" $rxDataAddr]

    set conversionData {}
    set adcData [mrd $rxDataAddr 8192]
    for { set i 1 } { $i < 16384 } { incr i 2} {
        set val [string trimleft [lindex $adcData $i] 0]
        if { ![string length $val] } {
            set val "0"
        }
        set val [expr 0x$val]
        lappend conversionData $val
        set addr [expr $rxDataAddr + 4]
        set rxDataAddr [format "0x%08x" $addr]
    }

puts "Ending the Microblaze program..."
    mwr $startAcqAddr 0
    con

# Create 16 lists, one for each channel
set final0 "Vin0"
set final1 "Vin1"
set final2 "Vin2"
set final3 "Vin3"
set final4 "Vin4"
set final5 "Vin5"
set final6 "Vin6"
set final7 "Vin7"
set final8 "Vin8"
set final9 "Vin9"
set final10 "Vin10"
set final11 "Vin11"
set final12 "Vin12"
set final13 "Vin13"
set final14 "Vin14"
set final15 "Vin15"

# Sorts data through channels and removes the channel number from the initial 4 bits
# If conding is 2's complement, process data
puts "Processing data"
for { set i 0 } { $i < [ llength $conversionData ] } { incr i} {
    set a [lindex $conversionData $i ]
    set decimal [ expr $a ]
    switch [ expr $decimal / 4096 ] {
        0   {
            if { $coding == 0 } {
                if { $decimal > 2047 } {
                    set decimal [expr $decimal - 4096]
                }
            }
            lappend final0 $decimal
        }
        1   {
            set decimal [expr $decimal - 4096 ]
            if { $coding == 0 } {
                if { $decimal > 2047 } {
                    set decimal [expr $decimal - 4096]
                }
            }
            lappend final1 $decimal
        }
        2   {
            set decimal [expr $decimal - 8192 ]
            if { $coding == 0 } {
                if { $decimal > 2047 } {
                    set decimal [expr $decimal - 4096]
                }
            }
            lappend final2 $decimal
        }
        3   {
            set decimal [expr $decimal - 12288 ]
            if { $coding == 0 } {
                if { $decimal > 2047 } {
                    set decimal [expr $decimal - 4096]
                }
            }
            lappend final3 $decimal
        }
        4   {
            set decimal [expr $decimal - 16384 ]
            if { $coding == 0 } {
                if { $decimal > 2047 } {
                    set decimal [expr $decimal - 4096]
                }
            }
            lappend final4 $decimal
        }
        5   {
            set decimal [expr $decimal - 20480 ]
            if { $coding == 0 } {
                if { $decimal > 2047 } {
                    set decimal [expr $decimal - 4096]
                }
            }
            lappend final5 $decimal
        }
        6   {
            set decimal [expr $decimal - 24576 ]
            if { $coding == 0 } {
                if { $decimal > 2047 } {
                    set decimal [expr $decimal - 4096]
                }
            }
            lappend final6 $decimal
        }
        7   {
            set decimal [expr $decimal - 28672 ]
            if { $coding == 0 } {
                if { $decimal > 2047 } {
                    set decimal [expr $decimal - 4096]
                }
            }
            lappend final7 $decimal
        }
        8   {
            set decimal [expr $decimal - 32768 ]
            if { $coding == 0 } {
                if { $decimal > 2047 } {
                    set decimal [expr $decimal - 4096]
                }
            }
            lappend final8 $decimal
        }
        9   {
            set decimal [expr $decimal - 36864 ]
            if { $coding == 0 } {
                if { $decimal > 2047 } {
                    set decimal [expr $decimal - 4096]
                }
            }
            lappend final9 $decimal
        }
        10  {
            set decimal [expr $decimal - 40960 ]
            if { $coding == 0 } {
                if { $decimal > 2047 } {
                    set decimal [expr $decimal - 4096]
                }
            }
            lappend final10 $decimal
        }
        11  {
            set decimal [expr $decimal - 45056 ]
            if { $coding == 0 } {
                if { $decimal > 2047 } {
                    set decimal [expr $decimal - 4096]
                }
            }
            lappend final11 $decimal
        }
        12  {
            set decimal [expr $decimal - 49152 ]
            if { $coding == 0 } {
                if { $decimal > 2047 } {
                    set decimal [expr $decimal - 4096]
                }
            }
            lappend final12 $decimal
        }
        13  {
            set decimal [expr $decimal - 53248 ]
            if { $coding == 0 } {
                if { $decimal > 2047 } {
                    set decimal [expr $decimal - 4096]
                }
            }
            lappend final13 $decimal
        }
        14  {
            set decimal [expr $decimal - 57344 ]
            if { $coding == 0 } {
                if { $decimal > 2047 } {
                    set decimal [expr $decimal - 4096]
                }
            }
            lappend final14 $decimal
        }
        15  {
            set decimal [expr $decimal - 61440 ]
            if { $coding == 0 } {
                if { $decimal > 2047 } {
                    set decimal [expr $decimal - 4096]
                }
            }
            lappend final15 $decimal
        }
        default {
            puts $decimal
            puts "ERR"
        }
    }
}

# clean intermediary variables
unset decimal
unset conversionData

# Arranging data by channel
# variable used to signal when no new data is available
set data_processed 1

puts "Arranging channels"
for { set i 0 } { $i < 8193  } { incr i} {
    if { [ llength $final0 ] > [ expr {$i ? $i : 1} ] } {
        lappend final [ lindex $final0 $i ],
        set data_processed 1
    }

    if { [ llength $final1 ] >  [ expr {$i ? $i : 1} ] } {
        lappend final [ lindex $final1 $i ],
        set data_processed 1
    }

    if { [ llength $final2 ] > [ expr {$i ? $i : 1} ] } {
        lappend final [ lindex $final2 $i ],
        set data_processed 1
    }

    if { [llength $final3] > [ expr {$i ? $i : 1} ] } {
        lappend final [lindex $final3 $i],
        set data_processed 1
    }

    if { [llength $final4] > [ expr {$i ? $i : 1} ] } {
        lappend final [lindex $final4 $i],
        set data_processed 1
    }

    if { [llength $final5] > [ expr {$i ? $i : 1} ] } {
        lappend final [lindex $final5 $i],
        set data_processed 1
    }

    if { [llength $final6] > [ expr {$i ? $i : 1} ] } {
        lappend final [lindex $final6 $i],
        set data_processed 1
    }

    if { [llength $final7] > [ expr {$i ? $i : 1} ] } {
        lappend final [lindex $final7 $i],
        set data_processed 1
    }

    if { [llength $final8] > [ expr {$i ? $i : 1} ] } {
        lappend final [lindex $final8 $i],
        set data_processed 1
    }

    if { [llength $final9] > [ expr {$i ? $i : 1} ] } {
        lappend final [lindex $final9 $i],
        set data_processed 1
    }

    if { [llength $final10] > [ expr {$i ? $i : 1} ] } {
        lappend final [lindex $final10 $i],
        set data_processed 1
    }

    if { [llength $final11] > [ expr {$i ? $i : 1} ] } {
        lappend final [lindex $final11 $i],
        set data_processed 1
    }

    if { [llength $final12] > [ expr {$i ? $i : 1} ] } {
        lappend final [lindex $final12 $i],
        set data_processed 1
    }

    if { [llength $final13] > [ expr {$i ? $i : 1} ] } {
        lappend final [lindex $final13 $i],
        set data_processed 1
    }

    if { [llength $final14] > [ expr {$i ? $i : 1} ] } {
        lappend final [lindex $final14 $i],
        set data_processed 1
    }

    if { [llength $final15] > [ expr {$i ? $i : 1} ] } {
        lappend final [lindex $final15 $i],
        set data_processed 1
    }
    if { $data_processed == 0 } { 
        break
    }
    lappend final \n
    set data_processed 0
}

puts "Moving data into file Acquisition.csv..."
    set fd [ open "Acquisition.csv" "a+" ]
    puts -nonewline $fd [ join $final ]
    close $fd

puts "Disconnecting from the processor..."
    stop
    #disconnect 0

puts "Data acquisition is complete. Data is stored in the file Acquisition.csv."

